package net.sf.saxon.dom;
import net.sf.saxon.om.NamespaceConstant;
import net.sf.saxon.event.Emitter;

import org.w3c.dom.*;

import javax.xml.transform.TransformerException;


/**
  * DOMEmitter is an Emitter that attaches the result tree to a specified Node in a DOM Document
  */

public class DOMEmitter extends Emitter
{
    protected Node currentNode;
    protected Document document;
    private boolean canNormalize = true;

    /**
    * Start of the document.
    */

    public void startDocument ()
    {

    }

    /**
    * End of the document.
    */

    public void endDocument ()
    {

    }


    /**
    * Start of an element.
    */

    public void startElement (int nameCode, int typeCode, int properties) throws TransformerException {
        String qname = namePool.getDisplayName(nameCode);
        String uri = namePool.getURI(nameCode);
        try {
            Element element = document.createElementNS(uri, qname);
            currentNode.appendChild(element);
            currentNode = element;
        } catch (DOMException err) {
            throw new TransformerException(err);
        }
    }

    public void namespace (int namespaceCode, int properties) throws TransformerException {
        try {
        	String prefix = namePool.getPrefixFromNamespaceCode(namespaceCode);
    		String uri = namePool.getURIFromNamespaceCode(namespaceCode);
    		Element element = (Element)currentNode;
            if (!(uri.equals(NamespaceConstant.XML))) {
                if (prefix.equals("")) {
                    element.setAttribute("xmlns", uri);
                } else {
                    element.setAttribute("xmlns:" + prefix, uri);
                }
            }
        } catch (DOMException err) {
            throw new TransformerException(err);
        }
    }

    public void attribute (int nameCode, int typeCode, CharSequence value, int properties)
    throws TransformerException {
        String qname = namePool.getDisplayName(nameCode);
        String uri = namePool.getURI(nameCode);
        try {
    		Element element = (Element)currentNode;
            element.setAttributeNS(uri, qname, value.toString());
        } catch (DOMException err) {
            throw new TransformerException(err);
        }
    }

    public void startContent() throws TransformerException {}

    /**
    * End of an element.
    */

    public void endElement () throws TransformerException {
		if (canNormalize) {
	        try {
	            currentNode.normalize();
	        } catch (Throwable err) {
	        	canNormalize = false;
	        }      // in case it's a Level 1 DOM
	    }

        currentNode = currentNode.getParentNode();

    }


    /**
    * Character data.
    */

    public void characters (CharSequence chars, int properties) throws TransformerException
    {
        try {
            Text text = document.createTextNode(chars.toString());
            currentNode.appendChild(text);
        } catch (DOMException err) {
            throw new TransformerException(err);
        }
    }


    /**
    * Handle a processing instruction.
    */

    public void processingInstruction (String target, CharSequence data, int properties)
        throws TransformerException
    {
        try {
            ProcessingInstruction pi =
                document.createProcessingInstruction(target, data.toString());
            currentNode.appendChild(pi);
        } catch (DOMException err) {
            throw new TransformerException(err);
        }
    }

    /**
    * Handle a comment.
    */

    public void comment (CharSequence chars, int properties) throws TransformerException
    {
        try {
            Comment comment = document.createComment(chars.toString());
            currentNode.appendChild(comment);
        } catch (DOMException err) {
            throw new TransformerException(err);
        }
    }

    /**
    * Set output destination
    */

    public void setNode (Node node) {
        currentNode = node;
        if (node instanceof Document) {
            document = (Document)node;
        } else {
            document = currentNode.getOwnerDocument();
        }
    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
