package net.sf.saxon.event;
import net.sf.saxon.charcode.UnicodeCharacterSet;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.TransformerException;

/**
  * This class generates TEXT output
  * @author <A HREF="mailto:michael.h.kay@ntlworld.com>Michael H. Kay</A>
  */

public class TEXTEmitter extends XMLEmitter {

    private String mediaType = "text/plain";

    /**
    * Start of the document.
    */

    public void startDocument () throws TransformerException
    {
        String mime = outputProperties.getProperty(OutputKeys.MEDIA_TYPE);
        if (mime!=null) {
            mediaType = mime;
        }

        String byteOrderMark = outputProperties.getProperty(SaxonOutputKeys.BYTE_ORDER_MARK);

        if ("yes".equals(byteOrderMark)) {
            try {
                writer.write('\uFEFF');
            } catch (java.io.IOException err) {
                // Might be an encoding exception; just ignore it
            }
        }

        if (characterSet==null) {
            characterSet = UnicodeCharacterSet.getInstance();
        }
        //if (characterSet instanceof UnicodeCharacterSet) {
        //    characterSet = new UnicodeCharacterSet(true);
        //}
        empty = true;
    }

    /**
    * Produce output using the current Writer. <BR>
    * Special characters are not escaped.
    * @param chars Character sequence to be output
    * @param properties bit fields holding special properties of the characters
    * @exception TransformerException for any failure
    */

    public void characters(CharSequence chars, int properties) throws TransformerException {
        if ((properties & ReceiverOptions.NO_SPECIAL_CHARS) == 0) {
            int badchar = testCharacters(chars);
            if (badchar != 0) {
                throw new TransformerException(
                        "Output character not available in this encoding (decimal " + badchar + ")");
            }
        }
        try {
            writer.write(chars.toString());
        } catch (java.io.IOException err) {
            throw new TransformerException(err);
        }
    }

    /**
    * Output an element start tag. <br>
    * Does nothing with this output method.
    * @param nameCode The element name (tag)
     * @param typeCode The type annotation
     * @param properties Bit fields holding any special properties of the element
    */

    public void startElement(int nameCode, int typeCode, int properties) {
        // no-op
    }

    public void namespace(int namespaceCode, int properties) {}

    public void attribute(int nameCode, int typeCode, CharSequence value, int properties) {}


    /**
    * Output an element end tag. <br>
    * Does nothing  with this output method.
    */

    public void endElement() {
        // no-op
    }

    /**
    * Output a processing instruction. <br>
    * Does nothing  with this output method.
    */

    public void processingInstruction(String name, CharSequence value, int properties) throws TransformerException {}

    /**
    * Output a comment. <br>
    * Does nothing with this output method.
    */

    public void comment(CharSequence chars, int properties) throws TransformerException {}

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
