package net.sf.saxon.instruct;
import net.sf.saxon.Controller;
import net.sf.saxon.ParameterSet;
import net.sf.saxon.expr.StaticProperty;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.om.NamePool;
import net.sf.saxon.value.SequenceType;
import net.sf.saxon.value.Type;

import javax.xml.transform.TransformerException;

/**
* An xsl:attribute-set element in the stylesheet.
*/

// Note, there is no run-time check for circularity. This is checked at compile time.

public class AttributeSet extends Instruction {

    private AttributeSet[] useAttributeSets;     // the value of the use-attribute-sets attribute, as supplied
    private Block body;
    private boolean needsStackFrame;

    public AttributeSet() {}

    public void initialize( AttributeSet[] useAttributeSets,
                            Block body,
                            boolean needsStackFrame ) {
        this.useAttributeSets = useAttributeSets;
        this.body = body;
        this.needsStackFrame = needsStackFrame;
    }

    /**
    * Get the name of this instruction for diagnostic and tracing purposes
    */

    public void setInstructionDetails(Controller controller, NamePool namePool, InstructionDetails details) {
        details.setInstructionName("attribute-set");
    }

    public SequenceType getResultType() {
        return new SequenceType(Type.ATTRIBUTE, Type.ANY_SIMPLE_TYPE, StaticProperty.CARDINALITY_ALLOWS_ZERO_OR_MORE);
    }

    public TailCall processLeavingTail(XPathContext context) throws TransformerException {
        // apply the content of any attribute sets mentioned in use-attribute-sets

        if (useAttributeSets != null) {
            AttributeSet.expand(useAttributeSets, context);
        }

        // Note, we can ignore the result of processLeavingTail() since an attribute set can't contain a tail call

        if (needsStackFrame) {
            Bindery bindery = context.getController().getBindery();
            bindery.openStackFrame(ParameterSet.EMPTY_PARAMETER_SET, null);
            body.processLeavingTail(context);
            bindery.closeStackFrame();
        } else {
            body.processLeavingTail(context);
        }
        return null;
    }

    protected static void expand(AttributeSet[] asets, XPathContext context) throws TransformerException {
        for (int i=0; i<asets.length; i++) {
            asets[i].processLeavingTail(context);
        }
    }
}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
