package net.sf.saxon.instruct;
import net.sf.saxon.Controller;
import net.sf.saxon.event.NoOpenStartTagException;
import net.sf.saxon.event.Receiver;
import net.sf.saxon.expr.Expression;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.om.Item;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.om.SequenceIterator;
import net.sf.saxon.om.NamePool;

import javax.xml.transform.TransformerException;


/**
* An xsl:copy-of element in the stylesheet.
*/

public class CopyOf extends Instruction {

    private Expression select;
    private boolean copyNamespaces;
    private boolean copyAnnotations;

    public CopyOf(  Expression select,
                    boolean copyNamespaces,
                    boolean copyAnnotations ) {
        this.select = select;
        this.copyNamespaces = copyNamespaces;
        this.copyAnnotations = copyAnnotations;
    }

    /**
    * Get the name of this instruction, for diagnostics and tracing
    */

    public void setInstructionDetails(Controller controller, NamePool namePool, InstructionDetails details) {
        details.setInstructionName("copy-of");
    }

    /**
    * Process this xsl:copy-of instruction
    * @param context the dynamic context for the transformation
    * @return null - this implementation of the method never returns a TailCall
    */

    public TailCall processLeavingTail(XPathContext context) throws TransformerException {

        Controller controller = context.getController();
    	Receiver out = controller.getReceiver();

        int whichNamespaces = (copyNamespaces ? NodeInfo.ALL_NAMESPACES : NodeInfo.NO_NAMESPACES);

    	SequenceIterator iter = select.iterate(context);
	    while (iter.hasNext()) {
    		Item item = iter.next();
    		if (item instanceof NodeInfo) {
    		    try {
    		        ((NodeInfo)item).copy(out, whichNamespaces, copyAnnotations);
    		    } catch (NoOpenStartTagException err) {
                    recoverableError("Cannot write a namespace or attribute node when no element start tag is open",
                                    controller);
                } catch (SkipInstructionException err) {
                    recoverableError(err.getMessage(), controller);
                }
    		} else {
    		    out.characters(item.getStringValue(), 0);
    		}
    	}
    	return null;
    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
