package net.sf.saxon.instruct;
import net.sf.saxon.Controller;
import net.sf.saxon.expr.Expression;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.om.Item;
import net.sf.saxon.om.SequenceIterator;
import net.sf.saxon.om.NamePool;
import net.sf.saxon.trace.TraceListener;

import javax.xml.transform.TransformerException;


/**
* Handler for xsl:for-each elements in stylesheet.
*/

public class ForEach extends Instruction {

    private Expression select = null;

    public ForEach( Expression select ) {
        this.select = select;
    }

    /**
    * Get the name of this instruction for diagnostic and tracing purposes
    * @return the string "xsl:for-each"
    */

    public void setInstructionDetails(Controller controller, NamePool namePool, InstructionDetails details) {
        details.setInstructionName("for-each");
    }

    public TailCall processLeavingTail(XPathContext context) throws TransformerException {
        Controller controller = context.getController();
        SequenceIterator savedIterator = controller.getCurrentIterator();
        SequenceIterator iter = select.iterate(context);

        Template savedTemplate = controller.getCurrentTemplate();
        controller.setCurrentTemplate(null);

//        if (sortKeys != null) {
//            //controller.setCurrentIterator(iter);
//            FixedSortKeyDefinition[] reducedSortKeys = new FixedSortKeyDefinition[sortKeys.length];
//            XPathContext xpc = context.newContext();
//            for (int s=0; s<sortKeys.length; s++) {
//                reducedSortKeys[s] = sortKeys[s].reduce(xpc);
//            }
//            iter = new SortedIterator(  xpc,
//                                        iter,
//                                        reducedSortKeys);
//        }
        controller.setCurrentIterator(iter);
        XPathContext innerContext = context.newContext();
        innerContext.setCurrentIterator(iter);

        if (controller.isTracing()) {
            TraceListener listener = controller.getTraceListener();
            while(iter.hasNext()) {
                Item item = iter.next();
                listener.startCurrentItem(item);
                processChildren(innerContext);
                listener.endCurrentItem(item);
            }
        } else {
            while(iter.hasNext()) {
                iter.next();
                processChildren(innerContext);
            }
        }
        controller.setCurrentIterator(savedIterator);
        controller.setCurrentTemplate(savedTemplate);
        return null;
    }


}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
