package net.sf.saxon.instruct;
import net.sf.saxon.expr.PromotionOffer;
import net.sf.saxon.expr.StaticContext;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.expr.StaticProperty;
import net.sf.saxon.value.SequenceType;
import net.sf.saxon.value.Type;
import net.sf.saxon.event.ReceiverOptions;
import net.sf.saxon.Controller;
import net.sf.saxon.om.NamePool;

import javax.xml.transform.TransformerException;

/**
* Handler for xsl:text elements in stylesheet. <BR>
*/

public class Text extends SimpleNodeConstructor {

    private int options;

    /**
    * Create an xsl:text instruction
    * @param disable set to true if output escaping is to be disabled
    */

    public Text(boolean disable) {
        this.options = (disable ? ReceiverOptions.DISABLE_ESCAPING : 0);
    }

    /**
    * Get the name of this instruction for diagnostic and tracing purposes
    */

    public void setInstructionDetails(Controller controller, NamePool namePool, InstructionDetails details) {
        details.setInstructionName("text");
    }

    public SequenceType getResultType() {
        return new SequenceType(Type.TEXT, Type.ITEM, StaticProperty.CARDINALITY_EXACTLY_ONE);
    }

    /**
    * Process this instruction, that is, produce a processing-instruction node in the
    * result sequence.
    * @param context the dynamic context of this transformation
    * @throws TransformerException if any non-recoverable dynamic error occurs
    * @return always returns null in this implementation
    */

    public TailCall processLeavingTail(XPathContext context) throws TransformerException {
        CharSequence value = expandChildren(context);
        if (value.length() != 0) {
            context.getController().getReceiver().characters(value, options);
        }
        return null;
    }

    public void promoteInst(PromotionOffer offer) {
        throw new UnsupportedOperationException("Text instruction cannot be used as an expression");
    }

    public void typeCheck(StaticContext env) {}


}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
