package net.sf.saxon.om;
import net.sf.saxon.xpath.XPathException;
import net.sf.saxon.value.Value;
import net.sf.saxon.value.AtomicValue;

/**
 * An Item is an object that can occur as a member of a sequence.
 * There are two kinds of Item: simple values, and nodes.
 */

public interface Item  {

    /**
     * Get the value of the item as a string
     *
     * @exception XPathException if the string value cannot be obtained. Some
     *     items have no string value, e.g. a QName
     * @return the string value of the item
     */

    public String getStringValue() throws XPathException;

    /**
     * Get the type of the item. This will be a value such as Type.INTEGER or Type.ATTRIBUTE
     *
     * @return an integer code identifying the type of the item, available as
     *     a constant net.sf.saxon.value.Type
     * @see net.sf.saxon.value.Type
     */

    public int getItemType();

    /**
     * Get the typed value of the item
     *
     * @exception XPathException where no typed value is available, e.g. for
     *     an element with complex content
     * @return the typed value of the item. In general this will be a
     *     sequence, though at present in Saxon the typed value is always a
     *     single atomic value
     */

    public Value getTypedValue() throws XPathException;


}



//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
