package net.sf.saxon.om;

import net.sf.saxon.expr.LastPositionFinder;
import net.sf.saxon.expr.ReversibleIterator;


/**
* SingletonIterator: an iterator over a sequence of zero or one values
*/

public class SingletonIterator implements AxisIterator,
        ReversibleIterator, LastPositionFinder {

    private Item value;
    private boolean gone;

    public SingletonIterator(Item value) {
        this.value = value;
        gone = (value==null);
    }

    public boolean hasNext() {
        return !gone;
    }

    public Item next() {
        gone = true;
        return value;
    }

    public Item current() {
        return value;
    }

    public int position() {
        return 1;
    }

    public int getLastPosition() {
        return (value==null? 0 : 1);
    }

    public SequenceIterator getAnother() {
        return new SingletonIterator(value);
    }

    public SequenceIterator getReverseIterator() {
        return new SingletonIterator(value);
    }

    public Item getValue() {
        return value;
    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
