package net.sf.saxon.pattern;
import net.sf.saxon.value.Type;

/**
  * NodeTest is an interface that enables a test of whether a node has a particular
  * name and kind. A NodeKindTest matches the node kind only.
  *
  * @author <A HREF="mailto:michael.h.kay@ntlworld.com>Michael H. Kay</A>
  */

public class NodeKindTest extends NodeTest {

	private int kind;

	public NodeKindTest(int nodeKind) {
		kind = nodeKind;
		switch (nodeKind) {
		    case Type.DOCUMENT:
		        originalText = "/";
		        break;
		    case Type.ELEMENT:
		    case Type.ATTRIBUTE:
		        originalText = "*";
		        break;
		    case Type.COMMENT:
		        originalText = "comment()";
		        break;
		    case Type.TEXT:
		        originalText = "text()";
		        break;
		    case Type.PROCESSING_INSTRUCTION:
		        originalText = "processing-instruction()";
		        break;
		    case Type.NAMESPACE:
		        originalText = "namespace()";
		        break;
		}
	}

    /**
    * Test whether this node test is satisfied by a given node
    * @param nodeKind The type of node to be matched
    * @param fingerprint identifies the expanded name of the node to be matched
    */

    public boolean matches(int nodeKind, int fingerprint, int annotation) {
        return (kind == nodeKind);
    }

    /**
    * Determine the default priority of this node test when used on its own as a Pattern
    */

    public final double getDefaultPriority() {
    	return -0.5;
    }

    /**
    * Determine the types of nodes to which this pattern applies. Used for optimisation.
    * @return the type of node matched by this pattern. e.g. Type.ELEMENT or Type.TEXT
    */

    public int getItemType() {
        return kind;
    }

    /**
     * Indicate whether this NodeTest is capable of matching text nodes
     */

    public boolean allowsTextNodes() {
        return kind == Type.TEXT;
    }
}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
