package net.sf.saxon.sort;

import net.sf.saxon.Controller;
import net.sf.saxon.om.ArrayListIterator;
import net.sf.saxon.om.Item;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.om.SequenceIterator;
import net.sf.saxon.pattern.Pattern;
import net.sf.saxon.value.AtomicValue;
import net.sf.saxon.xpath.XPathException;

import java.util.ArrayList;
import java.util.List;

/**
 * A GroupEndingIterator iterates over a sequence of groups defined by
 * xsl:for-each-group group-ending-with="x". The groups are returned in
 * order of first appearance.
 */

public class GroupEndingIterator implements GroupIterator {

    private SequenceIterator population;
    private Pattern endPattern;
    private Controller controller;
    private List currentMembers;
    private Item next = null;
    private Item current = null;
    private int position = 0;

    public GroupEndingIterator(SequenceIterator population, Pattern endPattern,
                                 Controller controller)
    throws XPathException {
        this.population = population;
        this.endPattern = endPattern;
        this.controller = controller;
        // the first item in the population always starts a new group
        if (population.hasNext()) {
            next = population.next();
        }
     }

     private void advance() throws XPathException {
         currentMembers = new ArrayList();
         currentMembers.add(current);

         next = current;
         while (next != null) {
             if (endPattern.matches((NodeInfo)next, controller)) {
                 if (population.hasNext()) {
                     next = population.next();
                     break;
                 } else {
                     next = null;
                 }
             } else {
                 if (population.hasNext()) {
                     next = population.next();
                     currentMembers.add(next);
                 } else {
                     next = null;
                 }
             }
         }
     }

     public AtomicValue getCurrentGroupingKey() {
         return null;
     }

     public SequenceIterator iterateCurrentGroup() {
         return new ArrayListIterator(currentMembers);
     }

     public boolean hasNext() throws XPathException {
         return next != null;
     }

     public Item next() throws XPathException {
         current = next;
         position++;
         advance();
         return current;
     }

     public Item current() {
         return current;
     }

     public int position() {
         return position;
     }

    public SequenceIterator getAnother() throws XPathException {
        return new GroupEndingIterator(population, endPattern,
                controller);
    }



}
