package net.sf.saxon.sort;

import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.om.SequenceIterator;
import net.sf.saxon.om.Item;
import net.sf.saxon.xpath.XPathException;
import net.sf.saxon.value.ObjectValue;
import net.sf.saxon.value.Value;

/**
 * A SortedTupleIterator is a modified SortedIterator. Whereas the sorted iterator
 * used by XSLT computes the sort key of each item in a sequence, using that item
 * as the context item, the SortedTupleIterator used by XQuery precomputes the sort
 * keys from scratch; they do not need to be a function of the item being sorted.
 */

class SortedTupleIterator extends SortedIterator {

    public SortedTupleIterator(XPathContext context, SequenceIterator base,
                               FixedSortKeyDefinition[] sortKeys) throws XPathException {
        super(context, base, sortKeys);
    }

    /**
     * Override the method that builds the array of values and sort keys.
     * @throws XPathException
     */

    protected void buildArray() throws XPathException {
        int allocated = 100;
        nodeKeys = new Object[allocated * recordSize];
        count = 0;

        // initialise the array with data

        while (base.hasNext()) {
            ObjectValue tupleObject = (ObjectValue)base.next();
            Item[] tuple = (Item[])tupleObject.getObject();
            if (count==allocated) {
                allocated *= 2;
                Object[] nk2 = new Object[allocated * recordSize];
                System.arraycopy(nodeKeys, 0, nk2, 0, count * recordSize);
                nodeKeys = nk2;
            }
            int k = count*recordSize;
            nodeKeys[k] = new ObjectValue(tuple[0]);
                // this is the "item" that will be returned by the TupleIterator.
                // In general it is actually a sequence, so we wrap it in an ObjectValue
                // It subsequently gets unwrapped by the MappingFunction applied to the
                // output of the SortedTupleIterator.
            for (int n=0; n<sortkeys.length; n++) {
                nodeKeys[k+n+1] = tuple[n+1];
            }
            nodeKeys[k+sortkeys.length+1] = new Integer(count);
            count++;
        }
    }
}
