package net.sf.saxon.sql;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.expr.RoleLocator;
import net.sf.saxon.expr.TypeChecker;
import net.sf.saxon.expr.StaticProperty;
import net.sf.saxon.style.XSLGeneralVariable;
import net.sf.saxon.instruct.Instruction;
import net.sf.saxon.instruct.GeneralVariable;
import net.sf.saxon.instruct.TailCall;
import net.sf.saxon.instruct.InstructionDetails;
import net.sf.saxon.value.SequenceType;
import net.sf.saxon.value.Type;
import net.sf.saxon.value.Cardinality;
import net.sf.saxon.xpath.XPathException;
import net.sf.saxon.Controller;
import net.sf.saxon.om.NamePool;

import javax.xml.transform.TransformerConfigurationException;


/**
* An sql:insert element in the stylesheet.
*/

public class SQLColumn extends XSLGeneralVariable {

    /**
    * Determine whether this node is an instruction.
    * @return false - it is not an instruction
    */

    public boolean isInstruction() {
        return false;
    }

    /**
    * Determine whether this type of element is allowed to contain a template-body
    * @return false: no, it may not contain a template-body
    */

    public boolean mayContainTemplateBody() {
        return false;
    }

    public void validate() throws TransformerConfigurationException {
        if (!(getParentNode() instanceof SQLInsert)) {
            compileError("parent node must be sql:insert");
        }
        select = typeCheck("select", select);
        try {
            RoleLocator role =
                new RoleLocator(RoleLocator.INSTRUCTION, "sql:column/select", 0);
            select = TypeChecker.staticTypeCheck(select,
                        new SequenceType(Type.ATOMIC, Type.ITEM, StaticProperty.CARDINALITY_EXACTLY_ONE),
                        false, role);

        } catch (XPathException err) {
            compileError(err);
        }
    }

    public Instruction compile() throws TransformerConfigurationException {
        ColumnInstruction inst = new ColumnInstruction();
        initializeInstruction(inst);
        return inst;
    }

    public String getColumnName() {
        return getAttributeValue("", "name");
    }

    protected static class ColumnInstruction extends GeneralVariable {

        public ColumnInstruction() {}

        public void setInstructionDetails(Controller controller, NamePool namePool, InstructionDetails details) {
            details.setInstructionName("column");
            details.setProperty("instruction-namespace", "java:/net.sf.saxon.sql.SQLElementFactory");
        }

        public TailCall processLeavingTail(XPathContext context) {
            return null;
        }
    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
