package net.sf.saxon.style;
import net.sf.saxon.instruct.Instruction;
import net.sf.saxon.instruct.Text;
import net.sf.saxon.tree.AttributeCollection;
import net.sf.saxon.value.Type;
import net.sf.saxon.value.StringValue;
import net.sf.saxon.om.AxisIterator;
import net.sf.saxon.om.Axis;

import javax.xml.transform.TransformerConfigurationException;

/**
* Handler for xsl:text elements in stylesheet. <BR>
*/

public class XSLText extends XSLStringConstructor {

    private boolean disable = false;

    /**
     * Determine the type of item returned by this instruction (only relevant if
     * it is an instruction).
     * @return the item type returned
     */

    protected int getReturnedItemType() {
        return Type.TEXT;
    }

    public void prepareAttributes() throws TransformerConfigurationException {

        String disableAtt = null;

		StandardNames sn = getStandardNames();
		AttributeCollection atts = getAttributeList();

		for (int a=0; a<atts.getLength(); a++) {
			int nc = atts.getNameCode(a);
			int f = nc & 0xfffff;
			if (f==sn.DISABLE_OUTPUT_ESCAPING) {
        		disableAtt = atts.getValue(a).trim();
        	} else {
        		checkUnknownAttribute(nc);
        	}
        }

        if (disableAtt != null) {
            if (disableAtt.equals("yes")) {
                disable = true;
            } else if (disableAtt.equals("no")) {
                disable = false;
            } else {
                compileError("disable-output-escaping attribute must be either yes or no");
            }
        }
    }

    public void validate() throws TransformerConfigurationException {
        checkWithinTemplate();

        // 2.0 spec has reverted to the 1.0 rule that xsl:text may not have child elements
        AxisIterator kids = iterateAxis(Axis.CHILD);
        while(kids.hasNext()) {
            if (kids.next() instanceof StyleElement) {
                compileError("xsl:text may not contain child elements");
                return;
            }
        }
        super.validate();
    }

    public Instruction compile() throws TransformerConfigurationException {
        Text inst = new Text(disable);
        if (select instanceof StringValue) {
            if (((StringValue)select).getStringValue().equals("")) {
                return null;    // empty xsl:text element is a no-op
            }
        }
        compileContent(inst);
        return inst;
    }

//    protected boolean getDisableOutputEscaping() {
//        return disable;
//    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
